package com.ensign.crm.module.system.dal.mysql.user;

import com.ensign.crm.framework.common.pojo.PageResult;
import com.ensign.crm.framework.mybatis.core.mapper.BaseMapperX;
import com.ensign.crm.framework.mybatis.core.query.LambdaQueryWrapperX;
import com.ensign.crm.module.system.controller.admin.user.vo.user.UserPageReqVO;
import com.ensign.crm.module.system.dal.dataobject.user.AdminUserDO;
import org.apache.ibatis.annotations.Mapper;

import java.util.Collection;
import java.util.List;

@Mapper
public interface AdminUserMapper extends BaseMapperX<AdminUserDO> {

    default AdminUserDO selectByUsername(String username) {
        return selectOne(AdminUserDO::getUsername, username);
    }

    default AdminUserDO selectByEmail(String email) {
        return selectOne(AdminUserDO::getEmail, email);
    }

    default AdminUserDO selectByMobile(String mobile) {
        return selectOne(AdminUserDO::getMobile, mobile);
    }

    default PageResult<AdminUserDO> selectPage(UserPageReqVO reqVO, Collection<Long> deptIds) {

        LambdaQueryWrapperX<AdminUserDO> queryWrapper = new LambdaQueryWrapperX<>();

        queryWrapper
                .likeIfPresent(AdminUserDO::getUsername, reqVO.getUsername())
                .likeIfPresent(AdminUserDO::getMobile, reqVO.getMobile())
                .eqIfPresent(AdminUserDO::getStatus, reqVO.getStatus())
                .betweenIfPresent(AdminUserDO::getCreateTime, reqVO.getCreateTime());

        if (deptIds != null && !deptIds.isEmpty()) {
            StringBuilder likeCondition = new StringBuilder();
            for (Long deptId : deptIds) {
                if (likeCondition.length() > 0) {
                    likeCondition.append(" OR ");
                }
                likeCondition.append("dept_id_coll LIKE '%").append(deptId).append("%'");
            }
            queryWrapper.and(q -> q.apply(likeCondition.toString()));
        }

        return selectPage(reqVO, queryWrapper.orderByDesc(AdminUserDO::getId));

//        return selectPage(reqVO, new LambdaQueryWrapperX<AdminUserDO>()
//                .likeIfPresent(AdminUserDO::getUsername, reqVO.getUsername())
//                .likeIfPresent(AdminUserDO::getMobile, reqVO.getMobile())
//                .eqIfPresent(AdminUserDO::getStatus, reqVO.getStatus())
//                .betweenIfPresent(AdminUserDO::getCreateTime, reqVO.getCreateTime())
//                .inIfPresent(AdminUserDO::getDeptId, deptIds)
//                .orderByDesc(AdminUserDO::getId));
    }

    default List<AdminUserDO> selectListByNickname(String nickname) {
        return selectList(new LambdaQueryWrapperX<AdminUserDO>().like(AdminUserDO::getNickname, nickname));
    }

    default List<AdminUserDO> selectListByStatus(Integer status) {
        return selectList(AdminUserDO::getStatus, status);
    }

    default List<AdminUserDO> selectListByDeptIds(Collection<Long> deptIds) {
        return selectList(AdminUserDO::getDeptId, deptIds);
    }

}
